' ########################################################################################
' Microsoft Windows
' File: CAfxMp3.inc
' Contents: MP3 class.
' Copyright (c) 2012 Jos Roca.
' Based on Simple C++ DirectShow MP3 Player Class by Wong Shao Voon, 26 Apr 2010.
' http://www.codeproject.com/Articles/373613/Simple-Cplusplus-DirectShow-MP3-Player-Class
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################

#Include This Once
%CAFXMP3_INC = 1

#Include Once "windows.inc"
#Include Once "DShow.inc"     ' // Direct Show

' ########################################################################################
' CAfxMp3 class.
' ########################################################################################
Class CAfxMp3

   Instance m_pIGraphBuilder As IGraphBuilder
   Instance m_pIMediaControl As IMediaControl
   Instance m_pIMediaEventEx As IMediaEventEx
   Instance m_pIBasicAudio As IBasicAudio
   Instance m_pIMediaSeeking As IMediaSeeking
   Instance m_ready As Long
   Instance m_duration As Quad

   Interface IAfxMp3 : Inherit IAutomation

   ' =====================================================================================
   ' Cleans resources
   ' =====================================================================================
   Method Cleanup
      m_pIGraphBuilder = Nothing
      m_pIMediaControl = Nothing
      m_pIMediaEventEx = Nothing
      m_pIBasicAudio = Nothing
      m_pIMediaSeeking = Nothing
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Builds a filter graph that renders the specified file.
   ' Return Value = %TRUE or %FALSE.
   ' OBJRESULT
   '   %S_OK = Success.
   '   %VFW_S_AUDIO_NOT_RENDERED = Partial success; the audio was not rendered.
   '   %VFW_S_DUPLICATE_NAME = Success; the Filter Graph Manager modified the filter name to avoid duplication.
   '   %VFW_S_PARTIAL_RENDER = Some of the streams in this movie are in an unsupported format.
   '   %VFW_S_VIDEO_NOT_RENDERED = Partial success; some of the streams in this movie are in an unsupported format.
   '   %E_ABORT = Operation aborted.
   '   %E_FAIL = Failure.
   '   %E_INVALIDARG = Argument is invalid.
   '   %E_OUTOFMEMORY = Insufficient memory.
   '   %E_POINTER = NULL pointer argument.
   '   %VFW_E_CANNOT_CONNECT = No combination of intermediate filters could be found to make the connection.
   '   %VFW_E_CANNOT_LOAD_SOURCE_FILTER = The source filter for this file could not be loaded.
   '   %VFW_E_CANNOT_RENDER = No combination of filters could be found to render the stream.
   '   %VFW_E_INVALID_FILE_FORMAT = The file format is invalid.
   '   %VFW_E_NOT_FOUND = An object or name was not found.
   '   %VFW_E_UNKNOWN_FILE_TYPE = The media type of this file is not recognized.
   '   %VFW_E_UNSUPPORTED_STREAM = Cannot play back the file: the format is not supported.
   ' =====================================================================================
   Method Load (ByVal bstrFile As WString) As Long
      Local hr As Long
      m_ready = %FALSE
      ME.Cleanup
      m_pIGraphBuilder = NewCom ClsId $CLSID_FilterGraph
      If IsNothing(m_pIGraphBuilder) Then Exit Method
      m_pIMediaControl = m_pIGraphBuilder
      m_pIMediaEventEx = m_pIGraphBuilder
      m_pIBasicAudio = m_pIGraphBuilder
      m_pIMediaSeeking = m_pIGraphBuilder
      hr = m_pIGraphBuilder.RenderFile(ByCopy bstrFile)
      If SUCCEEDED(hr) Then
         m_ready = %TRUE
         If IsObject(m_pIMediaSeeking) Then
            m_pIMediaSeeking.SetTimeFormat($TIME_FORMAT_MEDIA_TIME)   ' // Reference time (100-nanosecond units).
            m_pIMediaSeeking.GetDuration(m_duration)                  ' // Returns 10,000,000 for a second.
         End If
      End If
      Method = m_ready
      If hr Then Method ObjResult = hr
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Runs all the filters in the filter graph.
   ' Return Value = %TRUE or %FALSE.
   ' OBJRESULT
   '   %S_FALSE The graph is preparing to run, but some filters have not completed the transition to a running state.
   '   %S_OK = All filters in the graph completed the transition to a running state.
   ' =====================================================================================
   Method Run () As Long
      If IsTrue(m_ready) And IsObject(m_pIMediaControl) Then
         m_pIMediaControl.Run
         If IsFalse ObjResult Then Method = %TRUE Else Method ObjResult = ObjResult
      End If
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Pauses all the filters in the filter graph.
   ' Return Value = %TRUE or %FALSE.
   ' OBJRESULT
   '   %S_FALSE = The graph paused successfully, but some filters have not completed the state transition.
   '   %S_OK = All filters in the graph completed the transition to a paused state.
   ' =====================================================================================
   Method Pause () As Long
      If IsTrue(m_ready) And IsObject(m_pIMediaControl) Then
         m_pIMediaControl.Pause
         If IsFalse ObjResult Then Method = %TRUE Else Method ObjResult = ObjResult
      End If
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Stops all the filters in the filter graph.
   ' Return Value = %TRUE or %FALSE.
   ' OBJRESUT = Returns S_OK if successful, or an HRESULT value that indicates the cause of the error.
   ' =====================================================================================
   Method Stop () As Long
      If IsTrue(m_ready) And IsObject(m_pIMediaControl) Then
         m_pIMediaControl.Stop
         If IsFalse ObjResult Then Method = %TRUE Else Method ObjResult = ObjResult
      End If
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Waits for the filter graph to render all available data.
   ' Parameters:
   ' msTimeout
   '   [in] Duration of the time-out, in milliseconds. Pass zero to return immediately.
   '   To block indefinitely, pass INFINITE.
   ' EvCode
   '   [out] Event that terminated the wait. This value can be one of the following:
   '   %EC_COMPLETE  = Operation completed.
   '   %EC_ERRORABORT = Error. Playback cannot continue.
   '   %EC_USERABORT  = User terminated the operation.
   '   Zero (0) = Operation has not completed.
   ' Return Value = %TRUE or %FALSE.
   ' OBJRESULT
   '   %S_OK = Success.
   '   %E_ABORT = Time-out expired.
   '   %VFW_E_WRONG_STATE = The filter graph is not running.
   ' Remarks
   ' This method blocks until the time-out expires, or one of the following events occurs:
   '   %EC_COMPLETE
   '   %EC_ERRORABORT
   '   %EC_USERABORT
   ' During the wait, the method discards all other event notifications.
   ' If the return value is S_OK, the EvCode parameter receives the event code that ended
   ' the wait. When the method returns, the filter graph is still running. The application
   ' can pause or stop the graph, as appropriate.
   ' =====================================================================================
   Method WaitForCompletion (ByVal msTimeout As Long, ByRef EvCode As Long) As Long
      If IsTrue(m_ready) And IsObject(m_pIMediaEventEx) Then
         m_pIMediaEventEx.WaitForcompletion(msTimeout, EvCode)
         If IsFalse ObjResult Then Method = %TRUE Else Method ObjResult = ObjResult
      End If
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' Retrieves the volume (amplitude) of the audio signal.
   ' Return Value = The volume. Divide by 100 to get equivalent decibel value.
   '                For example, 10,000 is 100 dB.
   ' OBJRESULT
   '   %E_NOTIMPL = The filter graph does not contain an audio renderer filter.
   '                (Possibly the source does not contain an audio stream.)
   '   %S_OK = Success.
   ' =====================================================================================
   Property Get Volume () As Long
      If IsTrue m_ready And IsObject(m_pIBasicAudio) Then
         Property = m_pIBasicAudio.Volume
         If ObjResult Then Property ObjResult = ObjResult
      End If
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Sets the volume (amplitude) of the audio signal.
   ' nVol = Specifies the volume, as a number from 10,000 to 0, inclusive. Full volume is 0, and
   '        10,000 is silence. Multiply the desired decibel level by 100. For example, 10,000 = 100 dB.
   ' OBJRESULT
   '   %S_OK = Success.
   '   %E_FAIL = The underlying audio device returned an error.
   '   %E_INVALIDARG = The value of nVol is invalid.
   '   %E_NOTIMPL = The filter graph does not contain an audio renderer filter.
   '                (Possibly the source does not contain an audio stream.)
   ' =====================================================================================
   Property Set Volume (ByVal nVol As Long)
      If IsTrue m_ready And IsObject(m_pIBasicAudio) Then
         m_pIBasicAudio.Volume = nVol
         If ObjResult Then Property ObjResult = ObjResult
      End If
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Retrieves the balance of the audio signal.
   ' Return Value = The balance ranges from -10,000 to 10,000. The value -10,000 means the
   '                right channel is attenuated by 100 dB and is effectively silent. The
   '                value 10,000 means the left channel is silent. The neutral value is 0,
   '                which means that both channels are at full volume. When one channel is
   '                attenuated, the other remains at full volume.
   ' OBJRESULT
   '   %E_NOTIMPL = The filter graph does not contain an audio renderer filter.
   '                (Possibly the source does not contain an audio stream.)
   '   %S_OK = Success.
   ' =====================================================================================
   Property Get Balance () As Long
      If IsTrue m_ready And IsObject(m_pIBasicAudio) Then
         Property = m_pIBasicAudio.Balance
         If ObjResult Then Property ObjResult = ObjResult
      End If
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Sets the balance of the audio signal.
   ' nBalance = Specifies the balance. The value can range from -10,000 to 10,000. The value
   '            -10,000 means the right channel is attenuated by 100 dB and is effectively
   '            silent. The value 10,000 means the left channel is silent. The neutral value
   '            is 0, which means that both channels are at full volume. When one channel is
   '            attenuated, the other remains at full volume.
   ' OBJRESULT
   '   %S_OK = Success.
   '   %E_FAIL = The underlying audio device returned an error.
   '   %E_INVALIDARG = The value of nBalance is invalid.
   '   %E_NOTIMPL = The filter graph does not contain an audio renderer filter.
   '                (Possibly the source does not contain an audio stream.)
   ' =====================================================================================
   Property Set Balance (ByVal nBalance As Long)
      If IsTrue m_ready And IsObject(m_pIBasicAudio) Then
         m_pIBasicAudio.Balance = nBalance
         If ObjResult Then Property ObjResult = ObjResult
      End If
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Gets the duration of the stream, in 100-nanosecond units.
   ' Returns 10,000,000 for a second.
   ' =====================================================================================
   Property Get Duration () As Quad
      Property = m_duration
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Gets the current position, relative to the total duration of the stream.
   ' =====================================================================================
   Property Get CurrentPosition () As Quad
      Local hr As Long
      Local curPos As Quad
      If IsTrue m_ready And IsObject(m_pIMediaSeeking) Then
         hr = m_pIMediaSeeking.GetCurrentPosition(curPos)
         If SUCCEEDED(hr) Then Property = curPos Else Property ObjResult = hr
      End If
   End Property
   ' =====================================================================================

   ' =====================================================================================
   ' Sets the current position and the stop position.
   ' Return Value = %TRUE or %FALSE.
   ' Parameters:
   '   pCurrent = [in, out] Pointer to a variable that specifies the current position, in units of the current time format.
   '   pStop = [in, out] Pointer to a variable that specifies the stop time, in units of the current time format.
   '   bAbsolutePositioning = [in] %TRUE or %FALSE. IF %TRUE, the specified position is absolute.
   '                          IF %FALSE, the specified position is relative.
   ' OBJRESULT
   '   %S_FALSE = No position change. (Both flags specify no seeking.)
   '   %S_OK = Success.
   '   %E_INVALIDARG = Invalid argument.
   '   %E_NOTIMPL = Method is not supported.
   '   %E_POINTER = NULL pointer argument.
   ' =====================================================================================
   Method SetPositions (ByRef pCurrent As Quad, ByRef pStop As Quad, ByVal bAbsolutePositioning As Long) As Long
      Local hr As Long
      Local dwFlags As Dword
      If IsTrue m_ready And IsObject(m_pIMediaSeeking) Then
         If bAbsolutePositioning Then
            dwFlags = %AM_SEEKING_AbsolutePositioning Or %AM_SEEKING_SeekToKeyFrame
         Else
            dwFlags = %AM_SEEKING_RelativePositioning Or %AM_SEEKING_SeekToKeyFrame
         End If
         hr = m_pIMediaSeeking.SetPositions(pCurrent, dwFlags, pStop, dwFlags)
         If SUCCEEDED(hr) Then Method = %TRUE Else Method ObjResult = hr
      End If
   End Method
   ' =====================================================================================

   ' =====================================================================================
   ' reference to IMediatEventEx
   ' =====================================================================================
   Method GetIMediaEventExObject () As IMediaEventEx
      Method = m_pIMediaEventEx
   End Method
   ' =====================================================================================

   End Interface

End Class
' ########################################################################################
